/* -*- Mode: C++; tab-width: 8; indent-tabs-mode: nil; c-basic-offset: 4 -*- */
/* This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at http://mozilla.org/MPL/2.0/. */

#include "nsISupports.idl"

/**
 * Interface for an object that wants to gather "random"
 * data to be used for entropy purposes.
 */
%{C++

/*
 * If anyone wants to collect the entropy distributed by the
 * event handler, they'll have to implement this CONTRACTID
 */
#define NS_ENTROPYCOLLECTOR_CONTRACTID "@mozilla.org/security/entropy;1"
%}
/* Buffer type - for passing random data to the entropy
 * collector.
 */
[ptr] native buffer(void);

[uuid(6f883680-ab9d-11d4-9978-00b0d02354a0)]
interface nsIEntropyCollector : nsISupports
{

  /**
   * Add the following bytes to the pool of data to be used
   * in gathering entropy.
   */
  void randomUpdate(in buffer entropy, in long bufLen); 

};
