/* This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at http://mozilla.org/MPL/2.0/. */

#include "nsISupports.idl"

interface nsIDOMElement;

[scriptable, uuid(7B4A099F-6F6E-4565-977B-FB622ADBFF49)]
interface inIFlasher : nsISupports 
{
  attribute DOMString       color;
  attribute boolean         invert;
  attribute unsigned short  thickness;

  void drawElementOutline(in nsIDOMElement aElement);
  void repaintElement(in nsIDOMElement aElement);
  void scrollElementIntoView(in nsIDOMElement aElement);
};

