/* This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at http://mozilla.org/MPL/2.0/. */

#include "nsISupports.idl"

interface nsIDOMMozSmsManager;

[scriptable, uuid(7752e51c-68ba-48c7-9fb6-32e5fbdb4599)]
interface nsIDOMMozNavigatorSms : nsISupports
{
  readonly attribute nsIDOMMozSmsManager mozSms;
};
