/* -*- Mode: IDL; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
/* This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at http://mozilla.org/MPL/2.0/. */

#include "domstubs.idl"

interface nsIDOMOfflineResourceList;

[scriptable, uuid(f8bbf8c3-c47b-465a-a221-22824449f689)]
interface nsIDOMClientInformation : nsISupports
{
  /**
   * Web Applications 1.0 Browser State: registerContentHandler
   * Allows web services to register themselves as handlers for certain content
   * types.
   * http://whatwg.org/specs/web-apps/current-work/
   */
  void registerContentHandler(in DOMString mimeType, in DOMString uri, in DOMString title);
  void registerProtocolHandler(in DOMString protocol, in DOMString uri, in DOMString title);

  boolean mozIsLocallyAvailable(in DOMString uri, in boolean whenOffline);
};


